-- RPG Project

-- LÖVE Shortcuts
la  = love.audio
le  = love.event
lfs = love.filesystem
lf  = love.font
lg  = love.graphics
li  = love.image
lj  = love.joystick
lk  = love.keyboard
lma = love.math
lm  = love.mouse
lp  = love.physics
ls  = love.sound
lsy = love.system
lth = love.thread
lt  = love.timer
lw  = love.window

-- Library Load-In
sti = require 'libs.STI'
anim8 = require 'libs.anim8'
bump = require 'libs.bump'
require 'libs.tserial'
require 'libs.lovedebug'
--require 'libs.PassCodec'

-- Setup Modules
require 'objs.character'
require 'objs.floor'
require 'objs.card'
require 'objs.mms'
require 'maps'
require 'gamestates_controls'

if lsy.getOS() == "Android" or lsy.getOS() == "iOS" then mobile = true end

-- LOVE
function love.load()
  if lfs.exists("assets/music/dream_devourer.mp3") then
    bgm = la.newSource("assets/music/dream_devourer.mp3", "stream")
    bgm:play()
    bgmsource = "dd"
  end
  lsign = {lg.newImage("assets/splash/love-sign.png"), 0}
  lwhal = {lg.newImage("assets/splash/love-whale.png"), 0}
  splasha = .5
  splashb = 1
  splashc = 2.6
  splashd = 1
  splashing = true
  fadealpha = 0
  
  --[[maps.moveTo("test")
  conw = mms.createMsg({ msg = "Controls:\nArrows: Move\nz: Interact\nc: View Tile Collision Data\nShift + F8: Open LoveDebug", x = 20, y = 16, win = 2, follow = true, save = true, noarrow = true })
  print("If you are using a touchscreen computer, then try typing \"touchCon = true\" to use touch controls!")]]
end

function loadPart2()
  switch = { }
  --maps.loadedges()
  --bkgt = lg.newImage("assets/maps/back.png")
  winW = lg.getWidth()
  winH = lg.getHeight()
  lm.setVisible(false)
  gbacanvas = lg.newCanvas(480, 320)
  gbamode = false
  tx, ty = 0, 0
  edit = 0
  lk.setKeyRepeat(true)
  dpad_img = lg.newImage('assets/controls/transparentDark07.png')
  a_img = lg.newImage('assets/controls/transparentDark34.png')
  b_img = lg.newImage('assets/controls/transparentDark35.png')
  touchCon = false
  activecard = 1
  --maps.moveTo("nine_spread")
  maps.moveTo("blank")
  mms.createMenu({ id = "main", msg = "New Story\nView Card Game Mode\nOptions\nQuit", choices = 4 })
  --state = gamestates.card_game
  --activemenu = "cardgame"
  state = gamestates.menu
  activemenu = "main"
  collectgarbage()
end

function love.update(dt)
  if splashing then
    if lsign[2] < splashb then
      lsign[2] = lsign[2] + splasha * dt
      lwhal[2] = lwhal[2] + (splasha * 3/4) * dt
    end
    if lsign[2] > splashb then
      splashc = splashc - dt
    end
    if splashc <= 0 then
      splasha = splasha * 1.2
      splashb = 40
      splashd = splashd - dt
    end
    if splashd <= 0 then
      fadeout = true
      splashd = 1000000
    end
  end
  
  
  if not stopup then
    if (not freeze) and state == gamestates.game then char_obj.update(dt) end
    if map ~= nil then map:update(dt) end
  end
  if fadeout then
    if fadealpha + 5 > 255 then
      fadealpha = 255
      fadeout = false
      if trans ~= nil then
        maps.moveTo(trans[1], trans[2], trans[3])
        trans = nil
        fadein = true
      end
      if splashing then
        splashing = nil
        splasha = nil
        splashb = nil
        splashc = nil
        splashd = nil
        lsign = nil
        lwhal = nil
        loadPart2()
        fadein = true
      end
    else
      fadealpha = fadealpha + dt * 500
    end
  end
  if fadein then
    if fadealpha - 5 < 0 then
      fadealpha = 0
      fadein = false 
      stopup = false
    else
      fadealpha = fadealpha - dt * 500
    end
  end
  mms.update(dt)
end

function love.draw()
  if splashing then
    lg.draw(lsign[1], lg.getWidth()/1.6, lg.getHeight()/3, 0, lsign[2], lsign[2], lsign[1]:getWidth()/2, lsign[1]:getHeight()/2)
    lg.draw(lwhal[1], lg.getWidth()/3, lg.getHeight()/2, 0, lwhal[2], lwhal[2], lwhal[1]:getWidth()/2, lwhal[1]:getHeight()/2)
  end
  
  if gbamode then
    winW = 480
    winH = 320
    winHR = lg.getHeight()/winH
    lg.setColor(120, 120, 164, 255)
    lg.rectangle('fill', -10, -10, lg.getWidth() + 10, lg.getHeight() + 10)
    lg.setColor(255, 255, 255, 255)
    lg.setCanvas(gbacanvas)
  else
    winW = lg.getWidth()
    winH = lg.getHeight()
  end
  if chars["player"] ~= nil then tx, ty = (-chars["player"].x + winW/2 - 16), (-chars["player"].y + winH/2 - 16) end
  lg.push()
  if tx == nil then tx, ty = 0, 0 end
  lg.translate(tx, ty)
  --if bkgt ~= nil then lg.draw(bkgt, -640, -640) end
  --maps.drawedges()
  lg.setBackgroundColor(0, 0, 0, 0)
  if map ~= nil then 
    map:setDrawRange(-tx, -ty, lg.getWidth(), lg.getHeight())
    map:draw()
  end
  if debugging then
    map:bump_draw(world)
  end
  
  if debugging and not noplayer then
    lg.rectangle('line', chars["player"].x - 224, chars["player"].y - 144, 480, 320)
  end
  
  mms.draw()
  lg.pop()
  mms.drawFollowing()
  lg.setColor(0, 0, 0, fadealpha)
  lg.rectangle('fill', -20, -20, winW + 20, winH + 20)
  lg.setColor(255, 255, 255, 255)
  if gbamode then 
    lg.setCanvas()
    lg.draw(gbacanvas, 96, lg.getHeight(), -1.5708, lg.getHeight()/480)
  end
  
  --[[winW = 480
  winH = 320
  winHR = lg.getHeight()/winH
  lg.setColor(255, 255, 255, 255)
  
  lg.setCanvas(gbacanvas)
  if chars["player"] ~= nil then tx, ty = (-chars["player"].x + winW/2 - 16), (-chars["player"].y + winH/2 - 16) end
  lg.push()
  if tx == nil then tx, ty = 0, 0 end
  lg.translate(tx, ty)
  if bkgt ~= nil then lg.draw(bkgt, -640, -640) end
  maps.drawedges()
  lg.setBackgroundColor(0, 0, 0, 0)
  if map ~= nil then 
    map:setDrawRange(-tx, -ty, lg.getWidth(), lg.getHeight())
    map:draw()
  end
  if debugging then
    map:bump_draw(world)
  end
  
  if debugging and not noplayer then
    lg.rectangle('line', chars["player"].x - 224, chars["player"].y - 144, 480, 320)
  end
  
  mms.draw()
  lg.pop()
  mms.drawFollowing()
  lg.setColor(0, 0, 0, fadealpha)
  lg.rectangle('fill', -20, -20, winW + 20, winH + 20)
  lg.setColor(255, 255, 255, 255)
  lg.setCanvas()
  
  if gbamode then 
    lg.draw(gbacanvas, 96, lg.getHeight(), -1.5708, lg.getHeight()/480)
  else
    if lg.getWidth() < lg.getHeight() then
      lg.draw(gbacanvas, 0, 0, 0, lg.getWidth()/480)
    else
      lg.draw(gbacanvas, lg.getWidth()/2 - (480*(lg.getHeight()/320))/2, 0, 0, lg.getHeight()/320)
    end
  end]]
  
  lg.setColor(255, 255, 255, 255)
  
  if (mobile or touchCon) and dpad_img ~= nil then
    if gbamode then
      lg.draw(dpad_img, lg.getWidth() - 448, lg.getHeight() - 64, -1.5708, 1.4)
      lg.rectangle('line', lg.getWidth() - 448, lg.getHeight() - 219, 70, 84)
      lg.rectangle('line', lg.getWidth() - 292, lg.getHeight() - 219, 70, 84)
    else
      lg.draw(dpad_img, 64, lg.getHeight() - 210)
      lg.draw(a_img, lg.getWidth() - 240, lg.getHeight() - 160)
      lg.draw(b_img, lg.getWidth() - 160, lg.getHeight() - 240)
    end
  end
end

function love.quit()
  
end

function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end
