-- Character Object

char_obj = {}
chars = { }

local playerfilter = function(item, other)
  if debugging then return end
  if other.ct == "wall" or other.properties ~= nil and other.properties.ct == "wall" then return 'slide' end
end

local playercfilter = function(item, other)
  if debugging then return end
  if other.ct == "wall" or other.properties ~= nil and other.properties.ct == "wall" then return 'slide' end
  if other.ct == "nonpc" or other.properties ~= nil and other.properties.ct == "nonpc" then return 'cross' end
  if other.ct == "floor" then return 'cross' end
end

local charfilter = function(item, other)
  if other.ct == "wall" or other.properties ~= nil and other.properties.ct == "wall" then return 'slide' end
  if other.ct == "nonpc" or other.properties ~= nil and other.properties.ct == "nonpc" then return 'slide' end
end

function char_obj.new(newc)
  local newChar = {
    id = newc.name,
    type = "char",
    -- Movement info
    x = newc.x,
    y = newc.y,
    w = newc.width or 32,
    h = newc.height or 32,
    dx = newc.x,
    dy = newc.y,
    xt = newc.x / 32,
    yt = newc.y / 32,
    facing = "down",
    speed = 96,
    isMoving = false,
    mreq = false,
    distQueue = { },
    ct = newc.properties.collidable or "wall",
    ox = newc.properties.ox or 0,
    oy = newc.properties.oy or 6,
    idleRoute = newc.properties.route or "none",
    routecount = newc.properties.routecount or 0,
    rcorig = newc.properties.routecount or 0,
    sprite = newc.properties.sprite
  }
  
  if newc.name == "player" then
    newChar.filter = playerfilter
  else
    newChar.filter = charfilter
  end
  
  -- Graphics
  if newChar.sprite ~= nil then
    newChar.sprite_img = lg.newImage('assets/char_sprites/'..newChar.sprite..'.png')
    newChar.grid = anim8.newGrid(newChar.sprite_img:getWidth() / 4, newChar.sprite_img:getHeight() / 4, newChar.sprite_img:getWidth(), newChar.sprite_img:getHeight())
    newChar.animdown = anim8.newAnimation(newChar.grid('1-4', 3), 0.2)
    newChar.animright = anim8.newAnimation(newChar.grid('1-4', 2), 0.2)
    newChar.animup = anim8.newAnimation(newChar.grid('1-4', 1), 0.2)
    newChar.animleft = anim8.newAnimation(newChar.grid('1-4', 4), 0.2)
  end
  print(newChar.w .. ", " .. newChar.h)
  chars[newc.name] = newChar
  world:add(chars[newc.name], newChar.x, newChar.y, newChar.w, newChar.h)
end

function char_obj.move(cid, dir)
  if chars[cid] == nil then return end
  if wait and not chars[cid].eventing then return end
  if type(dir) == "table" then
    for i, d in pairs(dir) do
      if i == 1 then chars[cid].dir = d[1] chars[cid].dist = d[2]
      else table.insert(chars[cid].distQueue, d) end
    end
  else
    chars[cid].odir = chars[cid].dir
    chars[cid].dir = dir
  end
  --if cid == "player" then floorcheck = true end
end

function char_obj.stop(cid, dir)
  if chars[cid] == nil then return end
  if dir == nil then chars[cid].dir = 'stop' end
  if chars[cid].dir == dir then chars[cid].dir = 'stop' end
end

function char_obj.update(dt)
  delt = round(dt, 4)
  for i, c in pairs(chars) do
    if c.type == "char" then
      local dx, dy = 0, 0
      if c.isMoving and c.sprite ~= nil then
        c.animdown:update(dt)
        c.animright:update(dt)
        c.animup:update(dt)
        c.animleft:update(dt)
      end
      
      c.routecount = c.routecount - dt
      
      if (not c.eventing) then
        if type(c.idleRoute) == "table" and (not (#c.distQueue > 0)) then
          print(#c.distQueue)
          for i, d in pairs(c.idleRoute) do
            if i == 1 then c.dir = d[1] c.dist = d[2]
            else table.insert(c.distQueue, d) end
          end
        elseif c.idleRoute == "random" and c.routecount <= 0 then
          math.randomseed(os.time())
          local newDir = math.random(1,7)
          if newDir == 1 then char_obj.move(c.id, {{"right", 1}}) end
          if newDir == 2 then char_obj.move(c.id, {{"left", 1}}) end
          if newDir == 3 then char_obj.move(c.id, {{"down", 1}}) end
          if newDir == 4 then char_obj.move(c.id, {{"up", 1}}) end
          c.routecount = c.rcorig
        end
      end
      
      if c.id == "player" and math.ceil(c.x % 32) == 0 and math.ceil(c.y % 32) == 0 and floorcheck then char_obj.act(true) floorcheck = false end
      
      if not c.isMoving then
        c.xt = c.x / 32
        c.yt = c.y / 32
        if c.dir == 'right' then
          c.facing = "right"
          dx = 32
          if c.dist ~= nil then
            c.dist = c.dist - 1
          end
        end
        if c.dir == 'left' then
          c.facing = "left"
          dx = -32
          if c.dist ~= nil then
            c.dist = c.dist - 1
          end
        end
        if c.dir == 'down' then
          c.facing = "down"
          dy = 32
          if c.dist ~= nil then
            c.dist = c.dist - 1
          end
        end
        if c.dir == 'up' then
          c.facing = "up"
          dy = -32
          if c.dist ~= nil then
            c.dist = c.dist - 1
          end
        end
        if not world:hasItem(c) then print("Killing character update") return end
        local tdx, tdy, cols, cols_len = world:move(c, c.x + dx, c.y + dy, c.filter)
        if (tdx % 32) == 0 then c.dx = tdx end
        if (tdy % 32) == 0 then c.dy = tdy end
        if c.x ~= c.dx or c.y ~= c.dy then c.isMoving = true end
        if (not c.eventing) and c.mreq and cols_len > 0 and c.dist ~= nil then
          print(cols_len)
          c.dist = c.dist + 1
        end
        if c.dist ~= nil and c.dist == 0 then
          char_obj.stop(c.id, c.dir)
          c.dist = nil
          if #c.distQueue > 0 then c.dir = c.distQueue[1][1] c.dist = c.distQueue[1][2] table.remove(c.distQueue, 1) end
        end
      end
      
      if c.x < c.dx then
        c.x = math.ceil(c.x + c.speed * dt)
        if c.x > c.dx then c.x = c.dx end
      elseif c.x > c.dx then
        c.x = math.floor(c.x - c.speed * dt)
        if c.x < c.dx then c.x = c.dx end
      end
      if c.y < c.dy then
        c.y = math.ceil(c.y + c.speed * dt)
        if c.y > c.dy then c.y = c.dy end
      elseif c.y > c.dy then
        c.y = math.floor(c.y - c.speed * dt)
        if c.y < c.dy then c.y = c.dy end
      end
      if c.x == c.dx and c.y == c.dy and c.isMoving then c.isMoving = false if c.id == "player" then floorcheck = true end end
      --if c.x > c.dx - 2 and c.x < c.dx + 2 and c.y > c.dy - 2 and c.y < c.dy + 2 and c.id == "player" and floorcheck then char_obj.act(true) floorcheck = false end
      if c.dist == nil and c.waitOn and c.x == c.dx and c.y == c.dy then
        c.waitOn = false
        wait = false
        c.eventing = false
        if evoq ~= nil then
          maps.actdata[evoq]()
          evoq = nil
        end
      end
    end
  end
end

function char_obj.draw()
  local chardraw = { }
  for i, c in pairs(chars) do
    if not (#chardraw > 0) then
      table.insert(chardraw, c)
    else
      for k, t in pairs(chardraw) do
        if c.y < t.y then
          table.insert(chardraw, k-1, c)
          break
        else
          table.insert(chardraw, c)
          break
        end
      end
    end
  end
  for i, c in pairs(chardraw) do
    lg.setColor(255, 255, 255, 255)
    if c.sprite == nil and debugging then
      if c.type == "floor" then
        lg.setColor(0, 255, 0, 255)
      end
      lg.rectangle('line', c.x, c.y, c.w, c.h)
    elseif c.sprite == nil and c.type == "card" then
      lg.rectangle('line', c.x, c.y, c.w, c.h)
    elseif c.animdown ~= nil then
      if c.facing == "right" then c.animright:draw(c.sprite_img, c.x - c.ox, c.y - c.oy) end
      if c.facing == "left" then c.animleft:draw(c.sprite_img, c.x - c.ox, c.y - c.oy) end
      if c.facing == "down" then c.animdown:draw(c.sprite_img, c.x - c.ox, c.y - c.oy) end
      if c.facing == "up" then c.animup:draw(c.sprite_img, c.x - c.ox, c.y - c.oy) end
    elseif c.sprite ~= nil then
      if c.type == "card" and c.empty then
        lg.setColor(255, 255, 255, 164)
      else
        lg.setColor(255, 255, 255, 255)
      end
      lg.draw(c.sprite, c.x - c.ox, c.y - c.oy)
      if c.type == "card" then
        if c.id == activecard then
          tx, ty = -(c.x - winW/2 + c.w/2), -(c.y - winH/2 + c.h/2)
          lg.setColor(0, 120, 255, 255)
        else
          lg.setColor(255, 255, 255, 200)
        end
        lg.print(c.id, c.x, c.y - 24)
        lg.rectangle('line', c.x - c.ox - 4, c.y - c.oy - 4, c.w + 8, c.h + 8)
        local damtext = lg.newText(font[1])
        damtext:setf(c.dam[1].."\n"..c.dam[2].." "..c.dam[3].."\n"..c.dam[4], 100, "center")
        lg.setColor(0, 0, 0, 255)
        lg.draw(damtext, c.x - c.ox - 31, c.y - c.oy + 1)
        lg.setColor(255, 255, 255, 255)
        lg.draw(damtext, c.x - c.ox - 32, c.y - c.oy)
      end
    end
  end
end

function char_obj.act(ground)
  if chars["player"] == nil then return end
  local p = chars["player"]
  local cx, cy = p.x, p.y
  local cf = playerfilter
  if ground then cf = playercfilter -- Enables checking "floor" type objs
  elseif p.facing == "right" then cx = p.x + 32
  elseif p.facing == "left" then cx = p.x - 32
  elseif p.facing == "down" then cy = p.y + 32
  elseif p.facing == "up" then cy = p.y - 32 end
  local arbx, arby, chks, len = world:check(p, cx, cy, cf)
  for i = 1, len do
    local other = chks[i].other
    if not ground then print(other.id) end
    if other.isMoving then return end
    if maps.actdata ~= nil and maps.actdata[other.id] ~= nil then maps.actdata[other.id]() end
  end
end
